#if CP_PRAGMA_ONCE
#pragma once
#endif

#include "MCCompObj.h"
#include "Basic3DImportExport.h"
#include "MCBasicTypes.h"
#include "APITypes.h"
#include "IShParamUtilities.h"
#include "PublicUtilities.h"
#include "IShColorConversionUtilities.h"
#include "COMUtilities.h"
#include "COM3DUtilities.h"
#include "I3DShScene.h"
#include <math.h>
#include "Basic3DCOMImplementations.h"
#include "GenerateIvyDef.h"
#include "BitField.h"
#include "I3DShFacetMesh.h"
#include "I3DShScene.h"
#include "I3DExModifier.h"
#include "ISceneSelection.h"
#include "IChangeManagement.h"
#include "PublicUtilities.h"
#include "ISceneDocument.h"
#include "MCCountedPtrArray.h"
#include "MCCountedPtr.h"
#include "IShComponent.h" // for IShParameterComponent
#include "I3DShScene.h"
#include "I3DShGroup.h"
#include "I3DShInstance.h"
#include "ISHUtilities.h"
#include "MCBasicTypes.h"
#include "I3DShCamera.h"
#include "I3DShLightSource.h"
#include "I3DShObject.h"
#include "MCColor16.h"
#include "ColorRGB214.h"
#include "COMSafeUtilities.h"
#include "I3DShTreeElement.h"
#include "I3DShJoint.h"
#include "ISceneDocument.h"
#include "BonesUtilities.h"
#include "Transforms.h"
#include "I3DShModifier.h"
#include "IExtension.h"
#include "IFigureDataComponent.h"
#include "I3DExVertexPrimitive.h"
#include "I3DShUtilities.h"
#include "COM3DUtilities.h"
#include "I3DShMasterGroup.h"
#include "I3DShPrimitiveInstance.h"
#include "I3DShObject.h"
#include "I3DShGlobalDeformerHelper.h"
#include "I3DExPrimitive.h"
#include "IShComponent.h"
#include "TBBox.h"
#include "I3DExFigurePrimitive.h"
#include "I3DExPoserSkinningAlgorithm.h"
#include "I3DExSkinningAlgorithm.h"
#include "I3dExSkinable.h"
#include "I3DSHRenderFeature.h"
#include "IMFPart.h"
#include "IMFDialogPart.h"
#include "IShPartUtilities.h"
#include "MFPartTypes.h"
#include "I3DShShader.h"
#include "BonesUtilities.h"
#include "I3dExSkinable.h"
#include "UVMaps.h"
#include "I3DImageDocument.h"
#include "IShRasterLayer.h"
#include "IShRasterLayerUtilities.h"
#include "math.h"
#include "I3DShGroup.h"
#include "I3DShTreeElement.h"
#include "I3DShModifier.h"
#include "BonesUtilities.h"
#include "COMSafeUtilities.h"
#include "I3DShShadableTree.h"
#include "IComponentChooser.h"
#include "I3DShShader.h"
#include "I3DShAttributesSet.h"
#include "I3DShAttributesSetControler.h"
#include "I3DExAttributesSetControler.h"
#include "I3DExLinkedParamController.h"
#include "BasicAttributesSetControler.h"
#include "IExParamType.h"
#include "PMapTypes.h"
#include "I3dShAnimation.h"
#include "ParamHolder.h"
#include "IComponentChooser.h"
#include "Ivy.h"
#include "Vector2d.h"
#include "Vector3d.h"
#include "BasicMesh.h"
#include "Common.h"
#include "Transforms.h"
#include "BasicMCFCOMImplementations.h"
#include "IMFResponder.h"
#include "MFPartMessages.h"
#include <string>
#include <vector>


// define the SceneOp CLSID
extern const MCGUID CLSID_GenerateIvy;
extern const MCGUID CLSID_GenerateIvyDlg;



struct GenerateIvyData
{
	/** the ivy size factor, influences the grow behaviour [0..0,1] */
	real32 ivySize;

	/** leaf size factor [0..0,1] */
	real32 ivyLeafSize;

	/** branch size factor [0..0,1] */
	real32 ivyBranchSize;

    /** maximum length of an ivy branch segment that is freely floating [0..1] */
	real32 maxFloatLength;

	/** maximum distance for adhesion of scene object [0..1] */
	real32 maxAdhesionDistance;

	/** weight for the primary grow vector [0..1] */
	real32 primaryWeight;

	/** weight for the random influence vector [0..1] */
	real32 randomWeight;

	/** weight for the gravity vector [0..1] */
	real32 gravityWeight;

	/** weight for the adhesion vector [0..1] */
	real32 adhesionWeight;

	/** the probability of producing a new ivy root per iteration [0..1]*/
	real32 branchingProbability;

	/** the probability of creating a new ivy leaf [0..1] */
	real32 leafProbability;

	int32 growSteps;

	int32 leafDensityMultiplier;

	bool usePoserScale;

	GenerateIvyData() : 
		primaryWeight(0.5f),
		randomWeight(0.2f),
		gravityWeight(1.0f),
		adhesionWeight(0.1f),
		branchingProbability(0.99f),
		leafProbability(0.9f),
		ivySize(0.02f),
		ivyLeafSize(6.0f),
		ivyBranchSize(0.5f),
		maxFloatLength(1.5f),
		maxAdhesionDistance(3.0f),
		growSteps(300),
		leafDensityMultiplier(10),
		usePoserScale(false) {}

	GenerateIvyData(const GenerateIvyData &d) :
		primaryWeight(d.primaryWeight),
		randomWeight(d.randomWeight),
		gravityWeight(d.gravityWeight),
		adhesionWeight(d.adhesionWeight),
		branchingProbability(d.branchingProbability),
		leafProbability(d.leafProbability),
		ivySize(d.ivySize),
		ivyLeafSize(d.ivyLeafSize),
		ivyBranchSize(d.ivyBranchSize),
		maxFloatLength(d.maxFloatLength),
		maxAdhesionDistance(d.maxAdhesionDistance),
		growSteps(d.growSteps),
		leafDensityMultiplier(d.leafDensityMultiplier),
		usePoserScale(d.usePoserScale) {}
};


// SceneOp Object :
class GenerateIvyCommand : public TBasicSceneCommand
{
	public :  
		GenerateIvyCommand();
		~GenerateIvyCommand();

		STANDARD_RELEASE;

		// IExDataExchanger methods :
		virtual void*	 MCCOMAPI GetExtensionDataBuffer	() { return (&fData); }
		virtual int32	 MCCOMAPI GetParamsBufferSize		() const { return sizeof(GenerateIvyData); }
		virtual MCCOMErr MCCOMAPI ExtensionDataChanged		() { return MC_S_OK; }
		virtual int16	 MCCOMAPI GetResID					() { return 150; }

		virtual void	 MCCOMAPI GetMenuCallBack(ISelfPrepareMenuCallBack** callBack);

		virtual MCCOMErr MCCOMAPI Init						(ISceneDocument* sceneDocument);
		virtual MCCOMErr MCCOMAPI Prepare					();

		virtual boolean  MCCOMAPI CanUndo					();

		virtual boolean  MCCOMAPI Do						();
		virtual boolean  MCCOMAPI Undo						();
		virtual boolean  MCCOMAPI Redo						();

		Common myCommon;
		GenerateIvyData fData;

	private :
		TMCCountedPtr<ISceneDocument>	fSceneDocument;
		TMCCountedPtr<I3DShTreeElement>	fTree;
		TMCCountedPtr<I3DShScene>		fScene;
		TMCCountedPtr<IChangeChannel>	fSelectionChannel;
		TMCCountedPtr<IChangeChannel>	fTreePropertyChannel;
		TMCCountedPtr<ISceneSelection>	fSelection;
		TMCCountedPtr<ISceneSelection>	fCloneSelection;		// undo info

		void doIvy(I3DShTreeElement *growOnMe);
};




class GenerateIvyDlg : public TBasicPart
{
public :  
	GenerateIvyDlg() {};
	~GenerateIvyDlg() {};

	STANDARD_RELEASE;
	virtual uint32 MCCOMAPI AddRef () { return TBasicPart::AddRef(); }
	virtual MCErr MCCOMAPI QueryInterface(const MCIID& riid, void** ppvObj)
	{
		if (MCIsEqualIID(riid, CLSID_GenerateIvyDlg)) 
		{
			AddRef();
			*ppvObj = this;
			return MC_S_OK;
		}

		return TBasicPart::QueryInterface(riid, ppvObj);
	}

	virtual	boolean	MCCOMAPI Receive(int32 message, IMFResponder* source, void* data);
};

